/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.deployment;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import java.util.Vector;
import csoap.xml.rpc.handler.HandlerInfo2;
import javax.xml.namespace.QName;


public class SAXHandler extends DefaultHandler{
  private Vector services;
  private Vector undeployedServices;
  private Service service;
  private Operation operation;
  private HandlerInfo2 hand;
  private int stat = 0;
  private int tableStart[][] = new int[18][11];
  private int tableEnd[][] = new int[20][11];
  private Vector elements = new Vector(11);
  private Vector uris = new Vector();
  private Vector prefixs = new Vector();
  private boolean undeploy = false;
  
  
 public SAXHandler()
{
  services = new Vector();
  elements.addElement("deployment");
  elements.addElement("handler");
  elements.addElement("parameter");
  elements.addElement("service");
  elements.addElement("requestFlow");
  elements.addElement("responseFlow");
  elements.addElement("beanMapping");
  elements.addElement("typeMapping");
  elements.addElement("wsdd");
  elements.add("operation");
  elements.add("undeploy");

  tableStart[0][0]  = 1;
  tableStart[1][1]  = 2;
  tableStart[1][3]  = 3;
  tableStart[2][2]  = 4;
  tableStart[3][2]  = 5;
  tableStart[3][4]  = 6;
  tableStart[3][5]  = 7;
  tableStart[3][6]  = 8;
  tableStart[3][7]  = 9;
  tableStart[6][1]  = 10;
  tableStart[11][2] = 5;
  tableStart[11][5] = 7;
  tableStart[11][6] = 8;
  tableStart[11][7] = 9;
  tableStart[7][1]  = 12;
  tableStart[13][2] = 5;
  tableStart[13][6] = 8;
  tableStart[13][7] = 9;
  tableStart[13][9] = 15;
  tableStart[11][9] = 15;
  tableStart[3][9]  = 15;
  tableStart[15][2] = 16;
  tableStart[0][8]  = 0;
  tableStart[0][10] = 17;
  tableStart[17][3] = 18;
  


  tableEnd[2][1]   = 1;
  tableEnd[3][3]   = 14;
  tableEnd[4][2]   = 2;
  tableEnd[5][2]   = 13;
  tableEnd[6][4]   = 11;
  tableEnd[7][5]   = 13;
  tableEnd[8][6]   = 13;
  tableEnd[9][7]   = 13;
  tableEnd[10][1]  = 6;
  tableEnd[11][3]  = 14;
  tableEnd[12][1]  = 7;
  tableEnd[13][3]  = 14;
  tableEnd[14][0]  = 0;
  tableEnd[0][8]   = 100;
  tableEnd[15][9]  = 3;
  tableEnd[16][2]  = 15;
  tableEnd[18][3]  = 19;
  tableEnd[19][10] = 0;

  stat = 0;
}


  public void startDocument() throws SAXException
  {
      stat = 0;
      services = new Vector();
      undeployedServices = new Vector();
      undeploy = false;
  }

  public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
  {
    String qname="", prefix="";

    if (elements.indexOf(localName) !=-1) stat = tableStart[stat][elements.indexOf(localName)];
    else System.out.println("StartElement eligal");
    switch (stat)
    {
      case 1: service = new Service();
              break;
      case 2:
        try
        {
        hand = service.addHandlerInfo();
        Class handClass = Class.forName(atts.getValue("type"));
        hand.setHandlerClass(handClass);
        hand.setHandlerName(atts.getValue("name"));
        }
        catch (Exception ex)
        {
          ex.printStackTrace();
        }
        break;
      case 3:
	service.setServiceName(atts.getValue("name"));
        service.setServiceType(atts.getValue("scope"));
	service.setServiceProvider(atts.getValue("provider"));
        break;
      case 4:
        hand.addParameter(atts.getValue("name"), atts.getValue("value"));
        break;
      case 5:
        service.addParameter(atts.getValue("name"), atts.getValue("value"));
        break;
      case 8:
        qname = atts.getValue("qname");
        prefix = qname.substring(0,qname.indexOf(":"));
        if (prefixs.indexOf(prefix)!=-1) uri = (String)uris.elementAt(prefixs.indexOf(prefix));
        service.addBeanMapping(atts.getValue("qname"), uri ,atts.getValue("type"));
        //service.addBeanMapping(atts.getValue("qname"), uri ,atts.getValue("languageSpecificType"));
        break;
      case 9:
        qname = atts.getValue("qname");
        prefix = qname.substring(0,qname.indexOf(":"));
        if (prefixs.indexOf(prefix)!=-1) uri = (String)uris.elementAt(prefixs.indexOf(prefix));
        service.addTypeMapping(atts.getValue("qname"), uri ,atts.getValue("type"),atts.getValue("serializer"),atts.getValue("deserializer"),atts.getValue("encodingStyle"));
        //service.addTypeMapping(atts.getValue("qname"), uri ,atts.getValue("languageSpecificType"),atts.getValue("serializer"),atts.getValue("deserializer"),atts.getValue("encodingStyle"));
        break;
      case 10 :
       service.addRequestFlow(atts.getValue("type"));
       break;
      case 12 :
       service.addResponseFlow(atts.getValue("type"));
       break;
      case 15 :
        qname = atts.getValue("qname");
        QName Qname = null;
        if (qname != null)
        {
          if (qname.indexOf(":") != -1) {
            prefix = qname.substring(0, qname.indexOf(":"));
            qname = qname.substring(qname.indexOf(":") + 1);
            if (prefixs.indexOf(prefix) != -1)
              uri = (String) uris.elementAt(prefixs.indexOf(prefix));
            qname = "{" + uri + "}" + qname;
          }
          Qname = QName.valueOf(qname);
        }
        String returnQName = atts.getValue("returnQName");
        QName ReturnQName = null;
        if (returnQName != null)
        {
          if (returnQName.indexOf(":") != -1) {
            prefix = returnQName.substring(0, returnQName.indexOf(":"));
            returnQName = returnQName.substring(returnQName.indexOf(":") + 1);
            if (prefixs.indexOf(prefix) != -1)
              uri = (String) uris.elementAt(prefixs.indexOf(prefix));
            returnQName = "{" + uri + "}" + returnQName;
          }
          ReturnQName = QName.valueOf(returnQName);
        }
        String returnType = atts.getValue("returnType");
        QName ReturnType = null;

        if (returnType != null)
        {
          if (returnType.indexOf(":") != -1) {
            prefix = returnType.substring(0, returnType.indexOf(":"));
            returnType = returnType.substring(returnType.indexOf(":") + 1);
            if (prefixs.indexOf(prefix) != -1)
              uri = (String) uris.elementAt(prefixs.indexOf(prefix));
            returnType = "{" + uri + "}" + returnType;
          }
          ReturnType = QName.valueOf(returnType);          
        }
         operation = service.addOperation(atts.getValue("name"), Qname,ReturnQName, ReturnType);
        break;
      case 16:
        qname = atts.getValue("type");
        uri ="";
        if (qname.indexOf(":")!=-1)
        {
          prefix = qname.substring(0, qname.indexOf(":"));
          qname = qname.substring(qname.indexOf(":")+1);
          if (prefixs.indexOf(prefix)!=-1) uri = (String)uris.elementAt(prefixs.indexOf(prefix));
        }
        operation.addParameter(atts.getValue("name"),qname,uri, atts.getValue("mode"));
        break;
      case 17:
    	  service = new Service();
    	  undeploy = true;
    	  break;
      case 18:
    	  service.setServiceName(atts.getValue("name"));
    	  break;
      }

    }


  public void endElement(String uri, String localName, String qName) throws SAXException
  {
     if (elements.indexOf(localName)!=-1) stat = tableEnd[stat][elements.indexOf(localName)];
     if (stat == 0 && service != null) 
    	 {
    	 	if (undeploy) undeployedServices.addElement(service);
    	 	else services.addElement(service);
    	 }
   }

  public void endDocument() throws SAXException
  {
  }
  public void error(SAXParseException sAXParseException) throws SAXException
  {
    //System.out.println("Vous avez une erreur dans votre message SOAP");
  }
    public void fatalError(SAXParseException sAXParseException) throws SAXException
    {
    //System.out.println("Vous avez une erreur dans votre message SOAP");
    }
 public void startPrefixMapping(String prefix, String uri)
 {
  prefixs.insertElementAt(prefix,0);
  uris.insertElementAt(uri,0);
 }
    public Vector getServices()
    {
      return services;
    }
    
    public Vector getUndeployedService()
    {
    	return undeployedServices;
    }

}
