/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.deployment;

import java.util.*;
import javax.xml.namespace.QName;

public class Service extends handlerFlow
{
    private String serviceName="";
    private String serviceType="Request";     // request , application or session
    private String serviceProvider="JAVA:RPC"; // JAVA:RPC or JAVA:MSG  
    private Vector parameters = new Vector();

    private Vector operations = new Vector();
    private Vector operationQNames = new Vector();
    
    private Vector beanMappings = new Vector();
    private Vector typeMappings = new Vector();
    
    
    public Service(){}
    
    public Operation addOperation(String Name, QName Qname, QName ReturnQName, QName ReturnType)
    {
    	Operation operation = new Operation(Name, Qname, ReturnQName, ReturnType);
    	operations.addElement(operation);
    	operationQNames.addElement(Qname);
    	return operation;
    }
    
    public Operation getOperation(QName qname)
    {
    	if (operationQNames.indexOf(qname) !=-1 ) return (Operation)operations.elementAt(operationQNames.indexOf(qname));
    	return null;
    }
    
    public Iterator getOperations()
    {
    	return operations.iterator();
    }
    
    public void setServiceName(String serviceNameValue)
    {
    	serviceName = serviceNameValue;
    }
    
    public void setServiceType(String serviceTypeValue)
    {
        serviceType = serviceTypeValue;
    }
    
    public void setServiceProvider(String serviceProviderValue)
    {
    	if (serviceProviderValue!=null) serviceProvider = serviceProviderValue;
    }
    
    public String getServiceName()
    {
    	return serviceName;
    }
    
    public String getServiceType()
    {
    	return serviceType;
    }
    
    public String getServiceProvider()
    {
    	return serviceProvider;
    }

    public String getParameterValue(String parameterName)
    {
    	return (String)parameters.elementAt(parameters.indexOf(parameterName)+1);
    }
    
    public boolean isParameter(String parameterName)
    {
    	int x = -1;
    	x = parameters.indexOf(parameterName);
    	if (x % 2 == 0)return true;
		return false;
    }
    
    public void addParameter(String parameterName, String parameterValue)
    {
    	if (!isParameter(parameterName) )
	    {
			parameters.addElement(parameterName);
			parameters.addElement(parameterValue);
	    }
    	else System.out.println("Exisiting parameter");
    }
    
    public Vector getAllParameter()
    {
    	return parameters;
    }
    
    public void addBeanMapping(String qname, String uri, String type)
    {
    	beanMapping bean = new beanMapping(qname,uri,type);
    	beanMappings.addElement(bean);
    }
    
    public Iterator getBeanMappings()
    {
    	return beanMappings.iterator();
    }
    public beanMapping getBeanMapping(String qname, String uri)
    {
    	int i = 0;
    	boolean found = false;
    	if (qname.indexOf(":") !=-1 ) qname = qname.substring(qname.indexOf(":")+1);
    	while (i<beanMappings.size() && !found)
	    {
    		beanMapping bean = (beanMapping)beanMappings.elementAt(i);
    		if (bean.getLocalName().equals(qname) && bean.getURI().equals(uri)) found = true;
    		else i++;
	    }
    	if (found) return  (beanMapping)beanMappings.elementAt(i);
    	return null;
    }
    
    public beanMapping getBeanMapping(QName qname)
    {
    	int i = 0;
    	boolean found = false;
    	while (i<beanMappings.size() && !found)
	    {
    		beanMapping bean = (beanMapping)beanMappings.elementAt(i);
    		if (bean.getQualifiedName().equals(qname)) found = true;
    		else i++;
	    }
    	if (found) return (beanMapping)beanMappings.elementAt(i);
    	return null;
    }
	
    public beanMapping getBeanMapping(String javaType)
    {
    	int i = 0;
    	boolean found = false;
    	while (i<beanMappings.size() && !found)
	    {
    		beanMapping bean = (beanMapping)beanMappings.elementAt(i);
    		if (bean.getType().equals(javaType)) found = true;
				else i++;
	    }
    	if (found) return  (beanMapping)beanMappings.elementAt(i);
    	return null;
    }
    
    public void addTypeMapping(String qname, String uri, String type, String serializer, String deserializer, String encoding)
    {
    	typeMappingDeployed typeM = new typeMappingDeployed(qname,uri,type,serializer,deserializer,encoding);
    	typeMappings.addElement(typeM);
    }
    
    public typeMappingDeployed getTypeMapping(String qname, String uri)
    {
    	int i = 0;
    	boolean found = false;
    	if (qname.indexOf(":") !=-1 ) qname = qname.substring(qname.indexOf(":")+1);
    	while (i<typeMappings.size() && !found)
    	{
			typeMappingDeployed type = (typeMappingDeployed)typeMappings.elementAt(i);
			if (type.getLocalName().equals(qname) && type.getURI().equals(uri)) found = true;
			else i++;
    	}
    	if (found) return  (typeMappingDeployed)typeMappings.elementAt(i);
    	return null;
    }
    
    public typeMappingDeployed getTypeMapping(QName qname)
    {
    	int i = 0;
    	boolean found = false;
    	while (i<typeMappings.size() && !found)
	    {
    		typeMappingDeployed type = (typeMappingDeployed)typeMappings.elementAt(i);
    		if (type.getQualifiedName().equals(qname)) found = true;
    		else i++;
	    }
    	if (found) return (typeMappingDeployed)typeMappings.elementAt(i);
    	return null;
    }
    
    public Iterator getTypeMappings()
    {
    	return typeMappings.iterator();
    }
    
    public typeMappingDeployed getTypeMapping(String javaType)
    {
    	int i = 0;
    	boolean found = false;
    	while (i<typeMappings.size() && found)
	    {
    		typeMappingDeployed type = (typeMappingDeployed)typeMappings.elementAt(i);
    		if (type.getType().equals(javaType)) found = true;
    		else i++;
	    }
    	if (found) return (typeMappingDeployed)typeMappings.elementAt(i);
    	return null;
    }
    
}

