/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.deployment;

import java.util.*;
import java.io.*;
import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.io.Reader;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.XMLReader;
import javax.xml.namespace.QName;
import csoap.xml.rpc.encoding.*;
import javax.xml.rpc.encoding.*;
import csoap.xml.rpc.handler.HandlerInfo2;
import csoap.xml.rpc.handler.HandlerChainImpl;
import javax.xml.rpc.handler.HandlerRegistry;
import csoap.util.MapImpl;
import java.lang.reflect.*;


public class deployment {
    
    private TypeMappingRegistryImpl Registry;
    private HandlerRegistry handlerRegistry;
    private XMLReader xr;
    private SAXHandler sh;
    private String csoaphome=".";
    
    public deployment(deployedServices objectServices, 
                    TypeMappingRegistryImpl registry, 
                    HandlerRegistry hAndlerRegistry) {

    	 csoaphome = System.getProperty("csoap.home");
         if (csoaphome == null || csoaphome.equals("") || csoaphome.equals("null"))
        	 csoaphome = System.getenv("CSOAP_HOME");
         
         
         if (csoaphome == null || csoaphome.equals("") || csoaphome.equals("null") ) 
         {
         	System.out.println("CSOAP_HOME not defined");
         }
    	
        Registry = registry;
        handlerRegistry = hAndlerRegistry;
        
        xr = new SAXParser();
        sh = new SAXHandler();
        xr.setContentHandler(sh);
        xr.setErrorHandler(sh);

        
        
        readWsddFile(objectServices);
    }


    public int deploy(InputStream is, deployedServices objectServices) {
        InputSource inSource = new InputSource(is);
        return deploy(inSource, objectServices);
    }
    
    public int deploy(Reader reader, deployedServices objectServices) {
        InputSource inSource = new InputSource(reader);
        return deploy(inSource, objectServices);
    }
        
    private int deploy(InputSource is, deployedServices objectServices) {
        int forReturn = 0;
        
	try {
          xr.parse(is);
          Vector services = sh.getServices();
          Vector undeployedServices = sh.getUndeployedService();
          
          int resUndeploy = -1;
          for (int i=0; i<undeployedServices.size(); i++)
          {
        	  Service service = (Service)undeployedServices.elementAt(i);
        	  resUndeploy = objectServices.removeServiceNamed(service.getServiceName());
          }
          
          if (services.size()==0)
        	  {
        	  	writeWsddFile(objectServices);
        	  	return resUndeploy;
        	  }
          
          for (int i=0; i<services.size();i++)
            if (forReturn == 0)
              {
                Service service = (Service)services.elementAt(i); 
                //if (!service.getServiceName().equals("*")) forReturn = objectServices.addService(service);
                Iterator TypeMappingsDeployed = service.getTypeMappings();
                while (TypeMappingsDeployed.hasNext())
                {
                  typeMappingDeployed TypeMappingDeployed = (typeMappingDeployed)TypeMappingsDeployed.next();
                  QName xmlType = TypeMappingDeployed.getQualifiedName();
                  Class userClass;
                  String chaine = TypeMappingDeployed.getType();

                  if (chaine.indexOf("[") !=-1)
                  {
                	  chaine = chaine.substring(0,chaine.indexOf("["));
                	  if (chaine.equals("int")) userClass = int[].class;
                	  else
                	  {
                		  Class c = Class.forName(chaine);
                		  Object arrayObj = Array.newInstance(c, 1);
                		  userClass = arrayObj.getClass();
                	  }	
                  }
                  else userClass = Class.forName(chaine);
                  Class userSerializerClass = Class.forName(TypeMappingDeployed.getSerializer());
                  SerializerFactory userSerializerFactory = (SerializerFactory)userSerializerClass.getConstructor(new Class[] {Class.class,QName.class}).newInstance(new Object[]{userClass,xmlType});
                  Class userDeserializerClass = Class.forName(TypeMappingDeployed.getDeserializer());
                  DeserializerFactory userDeserializerFactory = (DeserializerFactory) userDeserializerClass.getConstructor(new Class[] {Class.class,QName.class}).newInstance(new Object[]{userClass,xmlType});
                  TypeMapping rpcTypeMapping = (TypeMapping) Registry.getTypeMapping(TypeMappingDeployed.getURI());
                  if (rpcTypeMapping != null)
                	  rpcTypeMapping.register(userClass,TypeMappingDeployed.getQualifiedName(),userSerializerFactory,userDeserializerFactory);
		  else
		      {
			  	rpcTypeMapping = Registry.createTypeMapping();
			  	rpcTypeMapping.register(userClass,TypeMappingDeployed.getQualifiedName(),userSerializerFactory,userDeserializerFactory);
			  	Registry.register(TypeMappingDeployed.getURI(), rpcTypeMapping);
		      }
               Registry.addMapping(userClass,TypeMappingDeployed.getQualifiedName());
                }
		
                HandlerChainImpl handlerChain = new HandlerChainImpl();
                Map configChain = new MapImpl();
                configChain.put("handlerFlow", service);
                handlerChain.init(configChain);

                Vector vect = new Vector();
                vect.addElement(handlerChain);

                handlerRegistry.setHandlerChain(QName.valueOf(service.getServiceName()),vect.subList(0,1));

                forReturn = objectServices.addService(service);
              }

          writeWsddFile(objectServices);
        }
        catch (Exception e)
        {
            e.printStackTrace();
          forReturn = -1;
        }
        return forReturn;
    }

 
 
 public void readWsddFile(deployedServices objectServices)
 {
   try
   {
	    String wsdlFilePath = csoaphome+System.getProperty("file.separator")+"configuration"+System.getProperty("file.separator")+"wsdd.conf";
        File wsdd_file = new File(wsdlFilePath);
        FileReader wsdd_read = new FileReader(wsdd_file);
        deploy(wsdd_read, objectServices);

   }
   catch (Exception e)
   {
       //System.out.println("readWsddFile: Error in reading wsdd_file");
     e.printStackTrace();
   }

 }

private void writeWsddFile(deployedServices objectServices)
 {
   
   try
   {
	   	String wsdlFilePath = csoaphome+System.getProperty("file.separator")+"configuration"+System.getProperty("file.separator")+"wsdd.conf";
        File wsdd_file = new File(wsdlFilePath);
        wsdd_file.delete();
        PrintStream wsdd_write = new PrintStream(new FileOutputStream(wsdd_file));
        wsdd_write.close();
        wsdd_write = new PrintStream(new FileOutputStream(wsdd_file));
        int index, indexparam;
        wsdd_write.println("<wsdd>");
        for (index=0;index<objectServices.services.size();index++)
        {
          wsdd_write.println("");
          Service service =(Service)objectServices.services.elementAt(index);
          wsdd_write.println("<deployment>");
          // handler
          Iterator handlers = service.getHandlers();
          while (handlers.hasNext())
          {
             HandlerInfo2 han = (HandlerInfo2)handlers.next();
             Iterator params = han.getParameters();
             if (params.hasNext())
              {
                wsdd_write.println("<handler name="+'"'+han.getHandlerName()+'"'+" type="+'"'+han.getHandlerClass().getName()+'"'+">");
                while (params.hasNext())
                {
                  parameter param = (parameter)params.next();
                  wsdd_write.println("<parameter name="+'"'+param.getName()+'"'+" value="+'"'+param.getValue()+'"'+"/>");
                 }
                wsdd_write.println("</handler>");
              }
             else wsdd_write.println("<handler name="+'"'+han.getHandlerName()+'"'+" type="+'"'+han.getHandlerClass().getName()+'"'+"/>");
           }

           wsdd_write.println("<service name="+'"'+service.getServiceName()+'"'+" scope="+'"'+service.getServiceType()+'"'+" provider="+'"'+service.getServiceProvider()+'"'+">");
           Iterator requests = service.getRequestFlow();
           if (requests.hasNext())
           {
             wsdd_write.println("<requestFlow>");
             while (requests.hasNext())
             {
               wsdd_write.println("<handler type="+'"'+(String)requests.next()+'"'+"/>");
             }
             wsdd_write.println("</requestFlow>");
           }

           Iterator responses = service.getResponseFlow();
           if (responses.hasNext())
           {
             wsdd_write.println("<responseFlow>");
             while (responses.hasNext())
             {
               wsdd_write.println("<handler type="+'"'+(String)responses.next()+'"'+"/>");
             }
             wsdd_write.println("</responseFlow>");
           }

           Vector parameters = service.getAllParameter();
           for( indexparam=0;indexparam<parameters.size();indexparam+=2)
           {
             String parameter = (String)parameters.elementAt(indexparam);
             String parameterValue = (String)parameters.elementAt(indexparam+1);
             wsdd_write.println("<parameter name="+'"'+parameter+'"'+" value="+'"'+parameterValue+'"'+"/>");
           }

           Iterator operations = service.getOperations();
           while (operations.hasNext())
           {
             Operation operation = (Operation)operations.next();
             wsdd_write.print("<operation name="+'"'+operation.getName()+'"');
             QName qname = operation.getQName();
             if (qname.getNamespaceURI()!=null && !qname.getNamespaceURI().equals(""))
             {
             wsdd_write.print(" qname="+'"'+"operNS:"+qname.getLocalPart()+'"'+" xmlns:operNS="+'"'+qname.getNamespaceURI()+'"');
             }
             else wsdd_write.print(" qname="+'"'+qname.getLocalPart()+'"');

            qname = operation.getReturnQName();
            if (qname != null)
              if (qname.getNamespaceURI()!=null && !qname.getNamespaceURI().equals(""))
            {
              wsdd_write.print(" returnQName="+'"'+"rqns:"+qname.getLocalPart()+'"'+" xmlns:rqns="+'"'+qname.getNamespaceURI()+'"');
            }
            else wsdd_write.print(" returnQName="+'"'+qname.getLocalPart()+'"');

            qname = operation.getReturnType();
            if (qname != null)
            if (qname.getNamespaceURI()!=null && !qname.getNamespaceURI().equals(""))
            {
              wsdd_write.print(" returnType="+'"'+"rtns:"+qname.getLocalPart()+'"'+" xmlns:rtns="+'"'+qname.getNamespaceURI()+'"');
            }
            else wsdd_write.print(" returnType="+'"'+qname.getLocalPart()+'"');

             Iterator params = operation.getParameters();
             if (params.hasNext())
             {
               wsdd_write.println(">");
               while (params.hasNext())
               {
                 parameterOperation param = (parameterOperation)params.next();
                 wsdd_write.println("<parameter name="+'"'+param.getName()+'"'+" type="+'"'+"tns:"+param.getQName().getLocalPart()+'"'+" xmlns:tns="+'"'+param.getQName().getNamespaceURI()+'"'+" mode="+'"'+param.getMode()+'"'+"/>");
               }
               wsdd_write.println("</operation>");
             }
             else wsdd_write.println("/>");
           }

           Iterator beanMappings = service.getBeanMappings();
           while (beanMappings.hasNext())
           {
             beanMapping bean = (beanMapping)beanMappings.next();
             wsdd_write.println("<beanMapping qname="+'"'+"ns:"+bean.getLocalName()+'"'+" xmlns:ns="+'"'+bean.getURI()+'"' +" languageSpecificType="+'"'+bean.getType()+'"'+"/>");
           }
           Iterator typeMappings = service.getTypeMappings();
           while (typeMappings.hasNext())
           {
             typeMappingDeployed typeM = (typeMappingDeployed)typeMappings.next();
             wsdd_write.println("<typeMapping qname="+'"'+"ns:"+typeM.getLocalName()+'"'+" xmlns:ns="+'"'+typeM.getURI()+'"' +" languageSpecificType="+'"'+typeM.getType()+'"'+" serializer="+'"'+typeM.getSerializer()+'"'+" deserializer="+'"'+typeM.getDeserializer()+'"'+" encodingStyle="+'"'+typeM.getEncodingStyle()+'"'+"/>");
           }

           wsdd_write.println("</service>");
           wsdd_write.println("</deployment>");
           wsdd_write.println("");

         }
         wsdd_write.println("</wsdd>");
         wsdd_write.close();

   }
   catch (Exception e)
   {
     e.printStackTrace();
   }


 }
}
