/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.server;

import java.util.*;
import java.lang.reflect.*;

import javax.xml.rpc.handler.soap.SOAPMessageContext;

public class RPCProvider{
    
    private Vector serviceNames = new Vector();   
    private Vector serviceClass = new Vector();
    private Vector lifecycle = new Vector(); // if value = I the class implements ServiceLifecycle Interface, else the class doesn't impelement ServiceLifecycle Interface


public Object invokeMethod(SOAPMessageContext msgContext,
                                  Method MyMethod, Object myObject,
                                  Object[] parameters)
    {
	try
	    {
			return MyMethod.invoke(myObject, parameters);
	    }
	catch (Exception ex)
	    {
			ex.printStackTrace();
			System.out.println("Error in Invoke method"+MyMethod.getName()+" Param "+parameters.toString()+" service class is "+myObject.getClass().getName());
	    }
	return null;
    }

public Object runService(SOAPMessageContext msgContext, 
                String className, String methodName, 
                Class[] parametersType, Object[] parameters, csoap.deployment.Service service) {
                    
        Object forReturn = null;                    
        try
        {
	    Object myObject = null;
	    boolean bool = false;
	    if (serviceNames.indexOf(service.getServiceName())==-1)
		{
		    Class MyClass = Class.forName(className);
		    myObject = MyClass.newInstance();
		    Class[] interfaces = MyClass.getInterfaces();
		    if (interfaces != null)
			{
			    int i=0;
			    while (!bool && i<interfaces.length)
				{
				    if (interfaces[i]!=null && interfaces[i].getName().equals("javax.xml.rpc.server.ServiceLifecycle")) bool = true;
				    i++;
				}
			    if (bool)
				{
				    lifecycle.add("I");
				    Method initMethod = myObject.getClass().getMethod("init", new Class[] {Object.class});
				    
				    initMethod.invoke(myObject, new Object[]{null});
				}
			    else  
				{
				    lifecycle.add("N");
				}
			}
		    else System.out.println("getClasses has return null");
		    serviceClass.addElement(myObject);
                    serviceNames.addElement(service.getServiceName());
		}
	    else myObject = serviceClass.elementAt(serviceNames.indexOf(service.getServiceName()));
	    Method MyMethod = myObject.getClass().getMethod(methodName, parametersType);
	    if (service.getServiceProvider().equals("JAVA:RPC"))
		forReturn = this.invokeMethod(msgContext, MyMethod ,myObject, parameters);
	    else 
		{
		    System.out.println(service.getServiceProvider()+" is used");
		     Class providerServiceClass =Class.forName(service.getServiceProvider()); 
		    csoap.server.ServiceProvider providerService = (csoap.server.ServiceProvider)providerServiceClass.newInstance();
		    forReturn = providerService.invokeMethod(msgContext,MyMethod, myObject, parameters);
		    providerService = null;
		    myObject= null;
		    System.gc(); 
		}
               if (service.getServiceType()!=null && service.getServiceType().equals("Request"))
	       { 
		 if (bool)
		     {
			 Method destroyMethod = serviceClass.elementAt(serviceNames.indexOf(service.getServiceName())).getClass().getMethod("destroy", new Class[] {});
			 if (destroyMethod != null) 
			     destroyMethod.invoke(serviceClass.elementAt(serviceNames.indexOf(service.getServiceName())),new Object[]{});
		     }
		 serviceClass.removeElementAt(serviceNames.indexOf(service.getServiceName()));
		 lifecycle.removeElementAt(serviceNames.indexOf(service.getServiceName()));
		 serviceNames.removeElement(service.getServiceName());		      }
	    }
         catch (Exception e) {
        	 //e.printStackTrace();
        	 //	throw new javax.xml.rpc.soap.SOAPFaultException(QName.valueOf("Service"),e.getMessage(),service.getServiceName(),null);
        	 System.out.println("Error in invok "+methodName+" under "+className+" class for cause ");
        	 e.printStackTrace();
        }
        return forReturn;
  }


    public void destroy()
    {
	try
	    {
		for (int i=0;i<serviceClass.size();i++)
		    {
			if (((String)lifecycle.elementAt(i)).equals("I"))
			    {
				Method destroyMethod = serviceClass.elementAt(i).getClass().getMethod("destroy", new Class[] {});
				if (destroyMethod != null && serviceClass.elementAt(i)!=null) 
				    destroyMethod.invoke(serviceClass.elementAt(i),new Object[]{});
			    }
		    }
		serviceClass.removeAllElements();
		lifecycle.removeAllElements();
		serviceNames.removeAllElements();
	    }
	catch (Exception ex)
	    {
		System.out.println("Error in destroy RPCProvider");
		ex.printStackTrace();
	    }

    }
     
}
