/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.rpc;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import javax.xml.rpc.*;
import java.util.Vector;
import csoap.xml.soap.*;
import csoap.xml.rpc.encoding.*;
import javax.xml.rpc.encoding.*;
import javax.xml.soap.*;
import javax.xml.rpc.soap.*;
import csoap.deployment.Service;
import csoap.util.*;

import javax.xml.rpc.handler.soap.SOAPMessageContext;
import csoap.xml.rpc.handler.soap.SOAPMessageContextImpl;
import javax.xml.rpc.handler.HandlerRegistry;
import javax.xml.rpc.handler.HandlerChain;

public class CallImpl implements Call
{
    
    public static final String USERNAME_PROPERTY = "javax.xml.rpc.security.auth.username";
    public static final String PASSWORD_PROPERTY = "javax.xml.rpc.security.auth.password";
    public static final String OPERATION_STYLE_PROPERTY = "javax.xml.rpc.soap.operation.style";
    public static final String SOAPACTION_USE_PROPERTY = "javax.xml.rpc.soap.http.soapaction.use";
    public static final String SOAPACTION_URI_PROPERTY = "javax.xml.rpc.soap.http.soapaction.uri";
    public static final String ENCODINGSTYLE_URI_PROPERTY = "javax.xml.rpc.encodingstyle.namespace.uri";
    public static final String SESSION_MAINTAIN_PROPERTY = "javax.xml.rpc.session.maintain";
    
    String endPoint;
    QName  returnXMLType, operationName, portType;
    Class  returnJAVAType;
    Vector parametersName = new Vector();
    Vector parametersXMLType = new Vector(30);
    Vector parameterPrefixs = new Vector(30);
    Vector parametersJAVAType = new Vector(30);
    Vector parametersMode = new Vector(30);
    Vector outputValues = new Vector(30);
    
    SOAPConnectionImpl      sOAPConnection = new SOAPConnectionImpl();
    TypeMappingRegistryImpl typeRegistry = new TypeMappingRegistryImpl();
    TypeMapping             typeMapping;
    DeserializerFactory     deserializerFactory;
    SerializerFactory       serializerFactory;
    DeserializerBase        deserializer;
    SerializerBase          serializer;
    
    Object result   = null;
    Service service = new Service();
   

    public CallImpl()
    {
	}
    
 // Mthodes
    
    public boolean isParameterAndReturnSpecRequired(QName xmlTtype)
    {
    	return true;
    }

    public void addParameter(String name, QName xmlType, ParameterMode parameterMode)
    {
    	parametersName.addElement(name);
    	parametersXMLType.addElement(xmlType);
    	parametersJAVAType.addElement(null);
    	parametersMode.addElement(parameterMode);
    }

    public void addParameter(String name, QName xmlType, Class javaType, ParameterMode parameterMode)
    {
    	parametersName.addElement(name);
    	parametersXMLType.addElement(xmlType);
    	parametersJAVAType.addElement(javaType);
    	parametersMode.addElement(parameterMode);
    }

    public QName getParameterTypeByName(String name)
    {
    	return (QName)parametersXMLType.elementAt(parametersName.indexOf(name));
    }
    
    public void setReturnType(QName xmlType)
    {
    	returnXMLType = xmlType;
    }
    
    public void setReturnType(QName xmlType, Class javaType)
    {
    	returnJAVAType = javaType;
    	returnXMLType = xmlType;
    }
    
    public QName getReturnType()
    {
    	return returnXMLType;
    }
    
    public void removeAllParameters()
    {
    	parametersName.removeAllElements();
    	parametersXMLType.removeAllElements();
    	parameterPrefixs.removeAllElements();
    	parametersJAVAType.removeAllElements();
    	parametersMode.removeAllElements();
    	returnXMLType=null;
    }
    
    public QName getOperationName()
    {
    	return operationName;
    }
    
    public void setOperationName(QName xmlOperation)
    {
    	operationName = xmlOperation;
    }
    
    public QName getPortTypeName()
    {
    	return portType;
    }
    
    public void setPortTypeName(QName PortType)
    {
    	portType = PortType;
    }
  
    public void setTargetEndpointAddress(String EndPoint) 
    {
    	endPoint = EndPoint;
    	service.setServiceName(EndPoint.substring(EndPoint.lastIndexOf("/")+1));
    }
  
    public String getTargetEndpointAddress() 
    {
    	return endPoint;
    }
  
    public void setProperty(String string, Object object)
    {
    	// Not implemented
    }
  
    public Object getProperty(String string)
    {
    	return new Object();
    }
  
    public void removeProperty(String string)
    {
    	// Not implemented
    }
    
    public Iterator getPropertyNames()
    {
    	return parametersMode.iterator();
    }
    
    public Object invoke(Object[] objectArray)
    {
    	SOAPMessageImpl message = new SOAPMessageImpl();
    	SOAPPartImpl sOAPPart = (SOAPPartImpl)message.getSOAPPart();
    	SOAPEnvelopeImpl sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
    	sOAPEnvelope.setEncodingStyle("http://schemas.xmlsoap.org/soap/encoding/");
    	sOAPEnvelope.addNamespaceDeclaration("xsi","http://www.w3.org/1999/XMLSchema-instance");
    	for (int i=0;i<parametersXMLType.size();i++)
    		sOAPEnvelope.addNamespaceDeclaration("ns"+(i+1),((QName)parametersXMLType.elementAt(i)).getNamespaceURI());
    	SOAPBodyImpl sOAPBody = (SOAPBodyImpl)sOAPEnvelope.getBody();
    	SOAPBodyElementImpl sOAPBodyElement = (SOAPBodyElementImpl)sOAPBody.addBodyElement(new NameImpl(operationName.getLocalPart(),"ns",operationName.getNamespaceURI()));
    
    	javax.xml.soap.SOAPElement sOAPElement; 
    	int j=0;
    	for (int i=0;i<parametersName.size();i++)
    	{
    		if (!((ParameterMode)parametersMode.elementAt(i)).equals(ParameterMode.OUT))
    		{	 
    			typeMapping = typeRegistry.getTypeMapping(((QName)parametersXMLType.elementAt(i)).getNamespaceURI());
    			Class c = null;
    			if (objectArray[j] != null) c = objectArray[j].getClass();
    			serializerFactory = typeMapping.getSerializer(c,(QName)parametersXMLType.elementAt(i));
    			serializer = (SerializerBase) serializerFactory.getSerializerAs("default");
    			sOAPElement = serializer.serialize(objectArray[j],new NameImpl((String)parametersName.elementAt(i)), sOAPBodyElement, typeRegistry);
    			sOAPBodyElement.addChildElement(sOAPElement);
    			j++;
    		}
    	}
    
    	if (message.saveRequired()) message.saveChanges();
    	message.saveChanges();
    	log.debug("SOAP Message built successfully", 3);
    
    	/*
    	 *  support for hanlder chain in making the call to service
    	 */       
    	SOAPMessageContext sOAPMessageContext = new SOAPMessageContextImpl();
    	sOAPMessageContext.setMessage(message);
    	sOAPMessageContext.setProperty("transport.url",endPoint);

    	
    	HandlerRegistry handlerRegistry = csoap.GlobalManagingClient.getHandlerRegistry(); 
    	// Handlerchain  for All
    	HandlerChain handlerChain;
    	List list = handlerRegistry.getHandlerChain(QName.valueOf("*"));
    	list = handlerRegistry.getHandlerChain(QName.valueOf("*"));

    	if (list != null) 
    	{
    		for (int i=0; i<list.size();i++) 
    		{
    			handlerChain = (HandlerChain)list.get(i);
    			handlerChain.handleRequest(sOAPMessageContext);    
    		}
    	}
    
        // Handlerchain for service
   
    	list = handlerRegistry.getHandlerChain(QName.valueOf(service.getServiceName()));
    	if (list != null)
    	{
    		for (int i=0; i<list.size();i++) 
    		{
    			handlerChain = (HandlerChain)list.get(i);
    			handlerChain.handleRequest(sOAPMessageContext);
    		}
    	}
    
    	/*
    	 *  making the call to service ...
    	 */
    	log.debug("Service runing ...", 2);
    	SOAPMessageImpl sOAPResponse = (SOAPMessageImpl)sOAPConnection.call(message,endPoint);

    	sOAPResponse.saveChanges();
    	sOAPMessageContext = new SOAPMessageContextImpl();
    	sOAPMessageContext.setMessage(sOAPResponse);
    	sOAPMessageContext.setProperty("transport.url",endPoint);

    	sOAPPart = (SOAPPartImpl)sOAPResponse.getSOAPPart();
    	sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
    	sOAPBody = (SOAPBodyImpl)sOAPEnvelope.getBody();

    	if (sOAPBody.hasFault())
    	{	
    		log.debug("Fault SOAP received", 2);
    		SOAPFaultImpl sOAPFault = (SOAPFaultImpl)sOAPBody.getFault();
    		throw new SOAPFaultException(QName.valueOf(sOAPFault.getFaultCode()),sOAPFault.getFaultActor(),sOAPFault.getFaultString(), null);
    	}
    	else
    	{
    		log.debug("Execution of the remote service successfully", 2);
    		Iterator bodyChilds = sOAPBody.getChildElements();
    		if (bodyChilds.hasNext()) {
    			sOAPBodyElement = (SOAPBodyElementImpl)bodyChilds.next();
    			Iterator methodChilds = sOAPBodyElement.getChildElements();
		
    			if (methodChilds.hasNext() && returnXMLType != null)
    			{

    				sOAPElement = (SOAPElement)methodChilds.next();
    				sOAPElement = Reference.getSOAPElementForDeserialiser(sOAPElement);
    				typeMapping = typeRegistry.getTypeMapping(returnXMLType.getNamespaceURI());
    				deserializerFactory = typeMapping.getDeserializer(null,returnXMLType);
    				deserializer = (DeserializerBase)deserializerFactory.getDeserializerAs("default");
    				result = deserializer.deserialize(sOAPElement,typeRegistry,service);
    			}
    			int i =0;
    			outputValues = new Vector();
    			while (methodChilds.hasNext())
    			{
    				if (!((ParameterMode)parametersMode.elementAt(i)).equals(ParameterMode.IN))
    				{
    					sOAPElement = (SOAPElement) methodChilds.next();
    					sOAPElement = Reference.getSOAPElementForDeserialiser(sOAPElement); 
    					typeMapping = typeRegistry.getTypeMapping(((QName)parametersXMLType.elementAt(i)).getNamespaceURI());
    					deserializerFactory = typeMapping.getDeserializer(null, (QName)parametersXMLType.elementAt(i));
    					deserializer = (DeserializerBase) deserializerFactory.getDeserializerAs("default");
    					Object objectResult = deserializer.deserialize(sOAPElement, typeRegistry,service);
    					outputValues.addElement(objectResult);
    				}
    				i++;
    			}          
				
    		}
    		else
    			log.debug("Response empty", 2);
    	}	
    	sOAPConnection.close();
    	return result;
    }
    
    public Object invoke(QName operationName, Object[] objectArray)
    {
    	return invoke(objectArray);
    }
    
    public void invokeOneWay(Object[] objectArray)
    {
    	SOAPMessageImpl message = new SOAPMessageImpl();
    	SOAPPartImpl sOAPPart = (SOAPPartImpl)message.getSOAPPart();
    	SOAPEnvelopeImpl sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
    	sOAPEnvelope.setEncodingStyle("http://schemas.xmlsoap.org/soap/encoding/");
    	sOAPEnvelope.addNamespaceDeclaration("xsi","http://www.w3.org/1999/XMLSchema-instance");
    	for (int i=0;i<parametersXMLType.size();i++)
    		sOAPEnvelope.addNamespaceDeclaration("ns"+(i+1),((QName)parametersXMLType.elementAt(i)).getNamespaceURI());
    	SOAPBodyImpl sOAPBody = (SOAPBodyImpl)sOAPEnvelope.getBody();
    	SOAPBodyElementImpl sOAPBodyElement = (SOAPBodyElementImpl)sOAPBody.addBodyElement(new NameImpl(operationName.getLocalPart(),"ns",operationName.getNamespaceURI()));
	
    	javax.xml.soap.SOAPElement sOAPElement; 
    	int j=0;
    	for (int i=0;i<parametersName.size();i++)
	    {
    		if (!((ParameterMode)parametersMode.elementAt(i)).equals(ParameterMode.OUT))
		    { 
    			typeMapping = typeRegistry.getTypeMapping(((QName)parametersXMLType.elementAt(i)).getNamespaceURI());
    			Class c = null;
    			if (objectArray[j] != null) c = objectArray[j].getClass();
    			serializerFactory = typeMapping.getSerializer(c,(QName)parametersXMLType.elementAt(i));
    			serializer = (SerializerBase) serializerFactory.getSerializerAs("default");
    			sOAPElement = serializer.serialize(objectArray[j],new NameImpl((String)parametersName.elementAt(i)), sOAPBodyElement, typeRegistry);
    			sOAPBodyElement.addChildElement(sOAPElement);
    			j++;
		    }
	    }
	
    	if (message.saveRequired()) message.saveChanges();
    	message.saveChanges();
    	log.debug("SOAP Message built successfully", 3);
    
    	/*
    	 *  support for hanlder chain in making the call to service
    	 */       
	
    	SOAPMessageContext sOAPMessageContext = new SOAPMessageContextImpl();
    	sOAPMessageContext.setMessage(message);
    	sOAPMessageContext.setProperty("transport.url",endPoint);
	
    	HandlerRegistry handlerRegistry = csoap.GlobalManagingClient.getHandlerRegistry(); 
    	// Handlerchain  for All
    	HandlerChain handlerChain;
    	List list = handlerRegistry.getHandlerChain(QName.valueOf("*"));
    	list = handlerRegistry.getHandlerChain(QName.valueOf("*"));
	
    	if (list != null) 
	    {
    		for (int i=0; i<list.size();i++) 
		    {
    			handlerChain = (HandlerChain)list.get(i);
    			handlerChain.handleRequest(sOAPMessageContext);    
		    }
	    }
	
	
	// Handlerchain for service
	
    	list = handlerRegistry.getHandlerChain(QName.valueOf(service.getServiceName()));
    	if (list != null)
	    {
    		for (int i=0; i<list.size();i++) 
    		{
    			handlerChain = (HandlerChain)list.get(i);
    			handlerChain.handleRequest(sOAPMessageContext);
    		}
	    }
	
    	/*
    	 *  making the call to service ...
    	 */
    	log.debug("Service runing ...", 2);
    	sOAPConnection.callOneWay(message,endPoint);
    	sOAPConnection.close();
    	//System.out.println("*************Service OneWay successfully run");
    }

    public Map getOutputParams()
    {
    	int j =0;
    	csoap.util.MapImpl map = new csoap.util.MapImpl();
    	for (int i=0; i< parametersName.size();i++)
	    {
    		if (!((ParameterMode)parametersMode.elementAt(i)).equals(ParameterMode.IN))
		    {
    			map.put(parametersName.elementAt(i), outputValues.elementAt(j));
    			j++;
		    }
	    }
    	return map;
    }

    public List getOutputValues()
    {
    	csoap.util.ListImpl list = new csoap.util.ListImpl();
    	for (int i=0; i< outputValues.size();i++)
    		list.add(outputValues.elementAt(i));
    	return list;
    }
    
    public void registerTypeMapping(Class javaClass , QName xmlType, SerializerFactory sf , DeserializerFactory df)
    {
    	service.addTypeMapping(xmlType.getLocalPart(), xmlType.getNamespaceURI(), javaClass.getName(),"", "","");
    	typeMapping = typeRegistry.getTypeMapping(xmlType.getNamespaceURI());
    	if (typeMapping != null) typeMapping.register(javaClass , xmlType, sf , df);
    	else
	    {
    		typeMapping = typeRegistry.createTypeMapping();
    		typeMapping.register(javaClass , xmlType, sf , df);
    		typeRegistry.register(xmlType.getNamespaceURI(),typeMapping);
	    }
    	typeRegistry.addMapping(javaClass, xmlType);
    }
 
}
