/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.rpc.encoding;

import java.util.Vector;
import csoap.xml.soap.*;
import java.lang.reflect.Constructor;
import javax.xml.namespace.QName;
import java.text.SimpleDateFormat;
import java.util.TimeZone;


public class DeserializerSimpleType implements DeserializerBase
{
    String mechanismType="default";
    Class JAVAClass = null;
    QName XMLType = null; 

    Vector primitiveType = new Vector(7);

    private static SimpleDateFormat zulu =
       new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                         //  0123456789 0 123456789

    static {
        zulu.setTimeZone(TimeZone.getTimeZone("GMT"));
    }


  public DeserializerSimpleType(Class JavaClass, QName XmlType)
  {
      JAVAClass = JavaClass;
      XMLType = XmlType;
      primitiveType.addElement(int.class);
      primitiveType.addElement(long.class);
      primitiveType.addElement(short.class);
      primitiveType.addElement(float.class);
      primitiveType.addElement(double.class);
      primitiveType.addElement(boolean.class);
      primitiveType.addElement(byte.class);
  }


  public Object deserialize(javax.xml.soap.SOAPElement soAPElement, javax.xml.rpc.encoding.TypeMappingRegistry typeMappingRegistry, csoap.deployment.Service service)
  {
    
       String isnull = (String)soAPElement.getAttributeValue(new NameImpl("nil"));
       if (isnull.equals("true")) return null;
       String value = soAPElement.getValue();
       Object obj = null;
       try
	   {
	     if (!JAVAClass.isPrimitive())  
		 {
		     // IF JAVACLASS NOT EQUALS java.util.Calendar
		     if(!JAVAClass.getName().equals(java.util.Calendar.class.getName()))
			 { 
			     Constructor constructor = JAVAClass.getConstructor(new Class[]{java.lang.String.class});
			     obj = constructor.newInstance(new Object[]{value});
			 }
		     else // Deserialise Calendar
			 {
			     java.util.Calendar objCalendar = java.util.Calendar.getInstance();
			     java.util.Date objDate = zulu.parse(value);
			     objCalendar.setTime(objDate);
			     obj = (Object)objCalendar;
			 }
		 }
	     else
		 {   Object tmp;
		     switch(primitiveType.indexOf(JAVAClass))
			 {
			 case 0 : tmp = new Integer(value);
			     return tmp;
			      
			 case 1 : tmp = new java.lang.Long(value); 
			     return tmp; 
			      
			 case 2 : tmp = new java.lang.Short(value);
			     return tmp; 
			      
			 case 3 : tmp = new java.lang.Float(value);
			     return tmp;
			      
			 case 4 : tmp = new java.lang.Double(value);
			     return tmp;
			      
			 case 5 : tmp = new java.lang.Boolean(value);
			     return tmp;
			      
			 case 6 : tmp = new java.lang.Byte(value);
			     return tmp;
			      
			 default : System.out.println("Primitive Type : "+JAVAClass.getName()+" not supported by CSOAP.");
			 }
		      
		 }
	   }
       catch(Exception ex)
	   {
    	       System.out.println("Error deserialiser simple type in create instance of :" +JAVAClass.getName());
	   }
    return obj;
  }

  public void setMechanismType(String MechanismType)
  {
    mechanismType = MechanismType;
  }

  public String getMechanismType()
  {
    return mechanismType;
  }
}
