/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.rpc.encoding;

import javax.xml.rpc.encoding.*;
import javax.xml.namespace.QName;
import java.util.Vector;

public class TypeMappingImpl implements TypeMapping {

    Vector supportedEncodings = new Vector();
    Vector serializerFactorys = new Vector();
    Vector deserializerFactorys = new Vector();
    Vector javaTypes = new Vector();
    Vector xmlTypes = new Vector();

    public TypeMappingImpl() { }

    public String[] getSupportedEncodings() 
    {
        String[] forReturn = new String[supportedEncodings.size()];
        for (int i =0 ; i< forReturn.length; i++)
        {
          forReturn[i] = (String)supportedEncodings.elementAt(i);
        }
        return forReturn;
    }
  
    public void setSupportedEncodings(String[] stringArray)
    {
        supportedEncodings.removeAllElements();
        for (int i=0;i<stringArray.length;i++) {
            supportedEncodings.addElement(stringArray[i]);
        }
    }
  
    public boolean isRegistered(Class javaType, QName xmlType) 
    {
    	return (getPosition(javaType,xmlType) !=-1);
    }
  
    public void register(Class javaType, QName xmlType, 
            SerializerFactory serializerFactory, 
            DeserializerFactory deserializerFactory) 
    {
    	javaTypes.addElement(javaType);
        xmlTypes.addElement(xmlType);
        serializerFactorys.addElement(serializerFactory);
        deserializerFactorys.addElement(deserializerFactory);
    }
  
    public SerializerFactory getSerializer(Class javaType, QName xmlType) 
    {
        int position = getPosition(javaType,xmlType);
        if (position !=-1) 
            return (SerializerFactory)serializerFactorys.elementAt(position);
        return null;
    }
  
    public DeserializerFactory getDeserializer(Class javaType, QName xmlType) 
    {
        int position = getPosition(javaType,xmlType);
        if (position !=-1) 
            return (DeserializerFactory)deserializerFactorys.elementAt(position);
        return null;
    }
  
    public void removeSerializer(Class javaType, QName xmlType) 
    {
        int index = getPosition(javaType,xmlType);
        serializerFactorys.removeElementAt(index);
        serializerFactorys.insertElementAt(null, index);
    }
    
    public void removeDeserializer(Class javaType, QName xmlType) 
    {
        int index = getPosition(javaType,xmlType);
        deserializerFactorys.removeElementAt(index);
        deserializerFactorys.insertElementAt(null, index);
    }
    
    private int getPosition(Class javaType, QName xmlType) 
    {
    	if (xmlType != null) 
    		return xmlTypes.indexOf(xmlType);
    	return javaTypes.indexOf(javaType);
    }
}
