/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.soap;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import java.util.Vector;

public class SAXHandler extends DefaultHandler {
  
    private SOAPMessageImpl  message;
    private SOAPPartImpl     sOAPPart;
    private SOAPEnvelopeImpl sOAPEnvelope;
    private SOAPHeaderImpl   sOAPHeader;
    private SOAPBodyImpl     sOAPBody;
    private SOAPElementImpl  currentElement;
    private SOAPFaultImpl    sOAPFault;
    
    private int stat = 0, cpt =0;
    long t = 0;
    
    Vector vecteur = new Vector(4);
    Vector prefixs = new Vector(10);
    Vector uris    = new Vector(10);
    

    
    public SAXHandler()
    {
        vecteur.addElement("Envelope");
        vecteur.addElement("Header");
        vecteur.addElement("Body");
        vecteur.addElement("Fault");

        message = new SOAPMessageImpl();
        sOAPPart = (SOAPPartImpl)message.getSOAPPart();
        sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
        stat = 0;
        cpt =0;
    }

    private void init()
    {
        message = new SOAPMessageImpl();
        sOAPPart = (SOAPPartImpl)message.getSOAPPart();
        sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
        stat = 0;
        cpt =0;
    }

    public SAXHandler(SOAPMessageImpl sOAPMessage) {
        vecteur.addElement("Envelope");
        vecteur.addElement("Header");
        vecteur.addElement("Body");
        vecteur.addElement("Fault");

        message = sOAPMessage;
        sOAPPart = (SOAPPartImpl)message.getSOAPPart();
        sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
        stat = 0;
        cpt =0;
    }

    public void startDocument() throws SAXException 
    {
        init();
    }

    
  public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
  {
	String prefix="";
    if (qName.indexOf(":") !=-1)
         prefix = qName.substring(0, qName.indexOf(":"));


    switch (vecteur.indexOf(localName))
    {
      case 0: if (stat == 0) {
                              stat = 1;
                              sOAPEnvelope.createName(localName, prefix, uri);
                              addNameSpaceDeclaration(sOAPEnvelope);
                              addAttributes(atts,sOAPEnvelope);
                              }
              else System.out.println("Envelope mal place");
              break;
      case 1:
        if (stat == 1) {
                         stat = 2;
                         sOAPHeader = (SOAPHeaderImpl)sOAPEnvelope.getHeader();
                         addNameSpaceDeclaration(sOAPHeader);
                         addAttributes(atts, sOAPHeader);
                        }
        else System.out.println("header mal plac");
        break;
      case 2:
        if ((stat == 1) || (stat == 5))
        {
          stat = 3;
          sOAPBody = (SOAPBodyImpl)sOAPEnvelope.getBody();
          addNameSpaceDeclaration(sOAPBody);
          addAttributes(atts, sOAPBody);
        }
        else System.out.println("Body mal plac");
        break;
      case 3:
        if (stat == 3)
        {
          stat = 7;
          sOAPFault = (SOAPFaultImpl)sOAPBody.addFault();
          addNameSpaceDeclaration(sOAPFault);
          addAttributes(atts, sOAPFault);
        }
        break;
      default:
        switch (stat)
        {
          case 3 :
          case 6 :
          case 8 : if (cpt==0) // 1er element
                   currentElement = (SOAPElementImpl)sOAPBody.addBodyElement(new NameImpl(localName,prefix,uri));
                   else currentElement = (SOAPElementImpl)currentElement.addChildElement(localName,prefix,uri);
                   addNameSpaceDeclaration(currentElement);
                   addAttributes(atts, currentElement);
                   stat = 6;
                   cpt++;
                   break;
          case 2 :
          case 4 : if (cpt==0) currentElement = (SOAPElementImpl)sOAPHeader.addHeaderElement(new NameImpl(localName,prefix,uri));
                   else currentElement = (SOAPElementImpl)currentElement.addChildElement(localName,prefix,uri);
                   addNameSpaceDeclaration(currentElement);
                   addAttributes(atts, currentElement);
                   stat = 4;
                   cpt++;
                   break;
          case 7 :
          case 10: if (cpt == 0) currentElement = (SOAPElementImpl)sOAPFault.addChildElement(localName, prefix, uri);
                   else currentElement = (SOAPElementImpl)currentElement.addChildElement(localName,prefix,uri);
                   addNameSpaceDeclaration(currentElement);
                   addAttributes(atts, currentElement);
                   stat = 10;
                   cpt++;
                   break;

          default : System.out.println("Element "+localName+" mal plac");
        }


    }
  }

  public void addNameSpaceDeclaration(SOAPElementImpl sOAPElement)
  {
    for (int i =0; i< prefixs.size();i++)
    sOAPElement.addNamespaceDeclaration((String)prefixs.elementAt(i),(String)uris.elementAt(i));
    prefixs.removeAllElements();
    uris.removeAllElements();
  }
 
  public void addAttributes(Attributes atts, SOAPElementImpl sOAPElement)
  {
    String prefix="";
    String qualifiedName="";
    for (int i = 0; i < atts.getLength();i++)
    {
      qualifiedName = atts.getQName(i);
      if (qualifiedName.indexOf(":") !=-1)
        {
          prefix = qualifiedName.substring(0, qualifiedName.indexOf(":"));
          sOAPElement.addAttribute(new NameImpl(atts.getLocalName(i),prefix), atts.getValue(i));
        }
      else sOAPElement.addAttribute(new NameImpl(atts.getQName(i)), atts.getValue(i));
    }
  }

  public void characters(char ch[], int start, int length)
  {
    if ((stat == 4 ) || (stat == 6 ) || (stat == 10 ))
    {
	String textNode = new String(ch, start, length);
	currentElement = (SOAPElementImpl) currentElement.addTextNode(textNode);
    }
  }
  public void endElement(String uri, String localName, String qName) throws SAXException
  {
	prefixs.removeAllElements();
    uris.removeAllElements();

    switch (vecteur.indexOf(localName))
    {
      case 0:
         if (stat == 9) stat = 0;
         else System.out.print("End Envelope mal plac");
        break;
      case 1:
        if (stat ==2)  stat = 1;
        else System.out.println("End header mal plac");
        break;
      case 2:
        if ((stat == 8) || (stat ==11)) stat = 9;
        else System.out.println("End Body mal plac");
        break;
      case 3:
        if (stat == 7) stat = 11;
        else System.out.println("enf Fault mal plac");
        break;
      default: switch (stat)
        {
          case 4 : cpt--;
              if (cpt == 0) stat =2;
              currentElement = (SOAPElementImpl)currentElement.getParentElement();
              break;
          case 6: cpt--;
             if (cpt==0) stat = 8;
             else currentElement = (SOAPElementImpl)currentElement.getParentElement();
             break;
          case 10: cpt--;
              if (cpt ==0) stat = 7;
              else currentElement = (SOAPElementImpl)currentElement.getParentElement();
              break;
          default : System.out.println("End Element mal plac");
        }

    }
  }
  public void startPrefixMapping(String prefix, String uri)
  {
    prefixs.addElement(prefix);
    uris.addElement(uri);
  }

  public void endPrefixMapping(String string)
  {
    // implementation is not needed
  }

  public void endDocument() throws SAXException
  {
	message.saveChanges();
  }
  
  public void error(SAXParseException sAXParseException) throws SAXException
  {
  }
  
  public void fatalError(SAXParseException sAXParseException) throws SAXException
  {
  }
  
  public SOAPMessageImpl getSOAPMessage()
  {
	return message;
  }
}
