/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.soap;

import javax.xml.namespace.QName;
import javax.xml.soap.*;

public class SOAPEnvelopeImpl extends SOAPElementImpl implements SOAPEnvelope {
    
    private SOAPBody sOAPBody; 
    private SOAPHeader sOAPHeader;
    private boolean body, header;

    public SOAPEnvelopeImpl()
    {   
        /*
         *  http://schemas.xmlsoap.org/soap/envelope
         *  is incompatible with axis
         *  must be used 
         *  http://schemas.xmlsoap.org/soap/envelope/
         */
        super("Envelope","SOAP","http://schemas.xmlsoap.org/soap/envelope/"); 
        try 
        {
            sOAPBody = addBody();
            sOAPHeader = addHeader(); 
            body = true;
            header = true;
        }
        catch (SOAPException ex) 
        {
            ex.printStackTrace();
        }
    }

    public SOAPEnvelopeImpl(String localName, String prefix, String uri)
    {
        super(localName,prefix,uri);
    }
    
    public SOAPEnvelopeImpl(String localName, String prefix)
    {
        super(localName,prefix);
    }
    
    public SOAPEnvelopeImpl(String prefix)
    {
        super("Envelope",prefix);
    }
    
    public SOAPEnvelopeImpl(QName qualifiedName)
    {
        super(qualifiedName);
    }

    public SOAPBody addBody() throws SOAPException 
    {
    	if (body)
            throw new SOAPException();
            
        sOAPBody = new SOAPBodyImpl(name.getPrefix());
        try {
            sOAPBody.setParentElement(this);
        } catch (javax.xml.soap.SOAPException ex) {
            body = false;
            return null;
        }
        /*super.*/childElements.addElement(sOAPBody);
        body = true;
        return sOAPBody;
    }

    public SOAPHeader addHeader() throws SOAPException  
    {
    	if (header) throw new SOAPException();
        sOAPHeader = new SOAPHeaderImpl(name.getPrefix());
        try
        {
            sOAPHeader.setParentElement(this);
        } 
        catch (javax.xml.soap.SOAPException ex) 
        {
            header = false;
            return null;
        }

        super.childElements.insertElementAt(sOAPHeader,0);
        header = true;
        return sOAPHeader;
    }
    
    public Name createName(String localName)
    {
        Name nameRet = new NameImpl(localName);
        return nameRet;
    }
    
    public Name createName(String localName, String prefix, String uri)
    {
        Name nameRet = new NameImpl(localName,prefix,uri);
        return nameRet;
    }
    
    public SOAPBody getBody() 
    {
        if (body) 
            return sOAPBody;
        else 
        return null;
    }

    public SOAPHeader getHeader() 
    {
        if (header) 
            return sOAPHeader;
        else 
            return null;
    }
}
