/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.soap;

import java.util.Iterator;
import java.util.Vector;
import javax.activation.DataHandler;
import java.io.*;
import javax.xml.soap.*;
 


public class SOAPMessageImpl extends javax.xml.soap.SOAPMessage
{
	private String             contentDescription; 
	private SOAPPartImpl       sOAPPart; 
    private Vector             attachmentParts;
    private MimeHeadersImpl    mimeHeaders;
    private boolean            saverequired; 
    private AttachmentPartImpl attachmentPart;
    private XMLWriter          xmlWriter; 


    public SOAPMessageImpl()
    {
        sOAPPart           = new SOAPPartImpl();
        attachmentParts    = new Vector();
        xmlWriter          = new XMLWriterImpl();
        contentDescription = "";
        saverequired       = false;
    }

  // Mthodes
  public String getContentDescription()
  {
    return contentDescription;
  }
  
  public void setContentDescription(String ContentDescription)
  {
      contentDescription = ContentDescription;
  }

  public SOAPPart getSOAPPart()
  {
     saverequired = true;
     return sOAPPart;
  }
  
  public void removeAllAttachments()
  {
	 attachmentParts.removeAllElements();
	 saverequired=true;
  }
  
  public int countAttachments()
  {
	 return attachmentParts.size();
  }
  
  public Iterator getAttachments()
  {
     return attachmentParts.iterator();
  }
  
  public Iterator getAttachments(javax.xml.soap.MimeHeaders mimeHeaders)
  { 
	// Faut comparer le contenu de mimeHeaders pas directement les objets parce que l'odre peut ne pas etr respecter
    Vector forReturn = new Vector();
    for (int i=0;i<attachmentParts.size();i++)
    {
      attachmentPart =(AttachmentPartImpl)attachmentParts.elementAt(i);
      if (attachmentPart.mimeHeaders.equals(mimeHeaders)) forReturn.addElement(attachmentPart);
    }
    return forReturn.iterator();
  }
  
  public void addAttachmentPart(AttachmentPart attachmentPart)
  {
    attachmentParts.addElement(attachmentPart);
    saverequired=true;
  }
  
  public AttachmentPart createAttachmentPart()
  {
    return new AttachmentPartImpl();
  }
  
  public AttachmentPart createAttachmentPart(DataHandler dataHandler)
  {
    attachmentPart = new AttachmentPartImpl();
    attachmentPart.setDataHandler(dataHandler);
    return attachmentPart;
  }
  
  public MimeHeaders getMimeHeaders()
  {
    return mimeHeaders;
  }
  
  public AttachmentPart createAttachmentPart(Object content, String contentType)
  {
    attachmentPart = new AttachmentPartImpl();
    attachmentPart.setContent(content, contentType);
    return attachmentPart;
  }
  
  public void saveChanges()
  { 
	SOAPEnvelopeImpl sOAPEnvelope = (SOAPEnvelopeImpl)sOAPPart.getEnvelope();
	xmlWriter.init();
	writeSOAPElement(sOAPEnvelope);
	saverequired=false;
  }	

  public void writeSOAPElement(SOAPElement element)
  {
    NameImpl name = (NameImpl)element.getElementName();
    xmlWriter.startElement(name.getLocalName(),name.getPrefix(),name.getURI());
    Iterator attributes = element.getAllAttributes();
    while (attributes.hasNext())
    {
      name = (NameImpl)attributes.next();
      xmlWriter.addAttribute(name.getQualifiedName(),element.getAttributeValue(name));
    }
    Iterator xmlns = element.getNamespacePrefixes();
    while (xmlns.hasNext())
    {
      String prefix = (String)xmlns.next();
      xmlWriter.addAttribute("xmlns:"+prefix,element.getNamespaceURI(prefix));
    }

    Iterator childs = element.getChildElements();
    if (childs.hasNext())
      while(childs.hasNext())
      {
        SOAPElement child = (SOAPElement)childs.next();
        writeSOAPElement(child);
      }
    else 
	{
	    xmlWriter.addTextNode(element.getValue());
	}
    xmlWriter.endElement();

  }

  public boolean saveRequired()
  {
    return saverequired;
  }

  public void writeTo(OutputStream outputStream)
  {
	  if (saveRequired()) saveChanges();
      xmlWriter.writeTo(outputStream);
  }
  
}
