/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package csoap.xml.soap;
import java.util.Vector;
import java.io.*;


public class XMLWriterImpl implements XMLWriter {
    
    Vector elements; 
    Vector stat; 
    Vector crush;
    boolean nullTextNode = false;
    
    public XMLWriterImpl() 
    {
        elements = new Vector();
        stat = new Vector();
        crush = new Vector();
    }

    public void init() 
    {
        elements = new Vector();
        stat = new Vector();
        crush = new Vector();
   }

  /// startElement(String localName, String prefix, String uri) util quand on veut ignorer les attributes
  public void startElement(String localName, String prefix, String uri)
  {
    String element ="";

    if(prefix!=null && !prefix.equals(""))
    	if (uri!=null && !uri.equals(""))
    	{
    		element = prefix + ":" + localName + " xmlns:" + prefix + "=" + '"' + uri + '"';
    		crush.addElement(prefix+":"+localName);
    	}
    	else
    	{
    		element = prefix + ":" + localName;
    		crush.addElement(prefix+":"+localName);
    	}
    else
    {
    	element = localName;
    	crush.addElement(localName);
    }

    elements.addElement(element);
    stat.addElement("E");
  }

  public void startElement(String localName, String prefix)
  {
	  
	  String element ="";
	  if(prefix!=null && !prefix.equals(""))
	  {
		  element = prefix + ":" + localName;
		  crush.addElement(prefix+":"+localName);
	  }
	  else
	  {
		  element = localName;
		  crush.addElement(localName);
	  }

	  elements.addElement(element);
	  stat.addElement("E");
  }

  public void addAttribute(String qName, String value)
  {
	  String element = (String)elements.lastElement();
	  element = element + " "+qName+"="+'"'+value+'"';
	  elements.setElementAt(element, elements.size()-1);
  }
  
  public void addTextNode(String textNode)
  {
	  nullTextNode= false;
	  if (textNode == null || textNode.equals("")) nullTextNode=true;
	  else
	  {
		  elements.addElement(textNode);
		  stat.addElement("T");
	  }
  }

  public void addBigTextNode(char[] bigTextNode)
  {
	  nullTextNode= false;
	  if (bigTextNode == null) nullTextNode=true;
	  else
	  {
		  int blocs = bigTextNode.length/256;
		  int i;
		  for (i=0;i<blocs;i++) addTextNode(new String(bigTextNode, i*256,256));
		  if (bigTextNode.length-(i*256)>0) addTextNode(new String(bigTextNode, i*256,bigTextNode.length-(i*256)));
	  }
  }

  public void endElement()
  {
	  if (nullTextNode)
	  {
		  elements.setElementAt((String) elements.lastElement() + "/", elements.size() - 1);
		  nullTextNode=false;
      }
	  else
	  {
		  elements.addElement("/" + (String) crush.lastElement());
		  stat.addElement("E");
	  }
	  crush.removeElementAt(crush.size()-1);
  }
  
  public String getSOAPMessage()
   {
	  String forReturn="";
	  for( int i=0; i<elements.size();i++)
         if (((String)stat.elementAt(i)).equals("E")) 
             forReturn = forReturn+"<"+elements.elementAt(i)+">";
         else 
             forReturn = forReturn+elements.elementAt(i);
	  return forReturn;
   }

  public void writeTo(OutputStream outputStream)
  {
	  try
	  {
		  for( int i=0; i<elements.size();i++)
			  if (((String)stat.elementAt(i)).equals("E"))
               outputStream.write(("<"+(String)elements.elementAt(i)+">").getBytes(),0,("<"+(String)elements.elementAt(i)+">").length());
			  else 
			  {
				  int len = ((String)elements.elementAt(i)).length();
				  int init =0;
				  String tosend =null;
				  while (len>1000)
				  {
					  tosend = ((String)elements.elementAt(i)).substring(init,init+1000);
					  outputStream.write(tosend.getBytes(),0,1000);
					  init = init +1000;
					  len= len-1000;
				  }
				  tosend = ((String)elements.elementAt(i)).substring(init,init+len);
				  outputStream.write(tosend.getBytes(),0,len);
	     }         
	  }
      catch(Exception ex)
      {
    	  ex.printStackTrace();
      }
  }
  
}
