/**
*
* CSoap Implementation, INRIA-Rocquencourt, ARLES Project
* 
* @author Rafik CHIBOUT 
* @version 1.0
*  
*/

package jetty.csoap;

import java.io.*;

import org.mortbay.http.*;
import org.mortbay.jetty.servlet.*;
import org.mortbay.http.handler.*;

public class MinimalJettyServer {
    
	public static final int STOPED  = 0;
    public static final int RUNNING = 1;
	public static final int TOSTOP  = 2;
    public static int HttpPort = 8080;
    
    
    public static void main (String[] args)
            throws Exception {      
                
        /*
        * jetty.home: used by CSoap to retrieve shared lib 
        *
        */
    	
    	String home = System.getProperty("csoap.home","");
    	if (home == null || home.equals("") || home.equals("null")) 
    	{
    		home = System.getenv("CSOAP_HOME");
    	}
    	
        if (home == null || home.equals("") || home.equals("null")) 
        {
        	System.out.println("CSOAP_HOME not defined");
        	return;
        }
        String  csoapBinDir=home+"/bin";
        
        if (args.length ==1 && args[0].equals("stop")) {
            stop(csoapBinDir);
            return;
        }

        if (args.length ==1) {
        	
        	String strPort = args[0];
        	try
        	{
        		HttpPort = Integer.valueOf(strPort).intValue();
        	}
        	catch (Exception ex)
        	{
        		System.out.println("HTTP Port must be a number");
        		return;	
        	}
            
        }
        
        // Create the server
        HttpServer server=new HttpServer();

        // Create a port listener
        SocketListener listener=new SocketListener();
        listener.setPort(HttpPort);
        server.addListener(listener);

        /*
        *      create root/servlet context
        */    
        HttpContext contextROOT = new HttpContext();
        contextROOT.setContextPath("/");

        // Serve static content from the context
        contextROOT.setResourceBase(home+"/public/");
        contextROOT.addHandler(new ResourceHandler());
        server.addContext(contextROOT);

        // Create a servlet container in root context  
        contextROOT.setClassPath(home+"/public/servlet/");
        ServletHandler servletsROOT = new ServletHandler();
        contextROOT.addHandler(servletsROOT);

        // Map a servlet into the container
        servletsROOT.addServlet("Invoker","/servlet/*",
        "org.mortbay.jetty.servlet.Invoker");

        /* end root/servlet context */


       
        /*
        *  Create csoap context 
        */
        HttpContext contextServices = new HttpContext();
        contextServices.setContextPath("/csoap");

        // Serve static content from the context
        contextServices.setResourceBase(home+"/csoap/");
        contextServices.setClassPath(home+"/csoap/WEB-INF/classes");
        contextServices.setClassPath(home+"/csoap/WEB-INF/lib");
        contextServices.addHandler(new ResourceHandler());
        server.addContext(contextServices);

        // Create a servlet container
        ServletHandler servletsServices = new ServletHandler();
        contextServices.addHandler(servletsServices);
        ServletHolder csoapServlet = servletsServices.addServlet ("CSoapServlet",  "/CSoapServlet", "csoap.CSoapServlet");
        servletsServices.addServlet ("CSoapServlet", "/*", "csoap.CSoapServlet");

        /* end csoap context */


        /*
        *  Start the http server
        */
        server.start ();
        // adminServlet.setInitOrder(1);
        // adminServlet.start();

        /*
        *      File .jetty_running is used to verify
        *      if the user has required to stop the
        *      http server and allow a clean stop
        *      and call destroy on CSoapServlet
        */
        FileOutputStream fos = new FileOutputStream(csoapBinDir+"/.jetty_running");
        int running = RUNNING+48; //48 is the char of '0'
        fos.write(running);
        fos.close();

        while (true) {
            try {                        
                Thread.sleep(20000);
                FileInputStream fis = new FileInputStream(csoapBinDir+"/.jetty_running");            
                running = fis.read(); 
                fis.close();
 
                if (running == TOSTOP+48) { //48 is the char of '0'        
                    server.stop();
                    fos = new FileOutputStream(csoapBinDir+"/.jetty_running");
                    
                    running = STOPED+48; //48 is the char of '0'
                    fos.write(running);
                    fos.close();
                    return;
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }       
        }
    }

    public static void stop(String csoapBinDir) {
        try {
            // read file value
        	FileInputStream fis = new FileInputStream(csoapBinDir+"/.jetty_running");            
            int running = fis.read(); 
            fis.close();

            if (running == RUNNING+48) { //48 is the char of '0'        
            	FileOutputStream fos = new FileOutputStream(csoapBinDir+"/.jetty_running");
            	running = TOSTOP+48; //48 is the char of '0'
                fos.write(running);
                fos.close();
                System.out.println("Stoping CSoap Server....");
                while (running != STOPED+48)
                {
                	fis = new FileInputStream(csoapBinDir+"/.jetty_running");            
                    running = fis.read(); 
                    fis.close();
                }
                System.out.println("CSoap Server is successfully Stoped");
            }
            else
            {
            System.out.println("CSoap Server is not running");	
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }   
    }
    
  

}
